@@ -42,11 +42,15 @@
 #endif
 
 #include "clutter-gst-video-sink.h"
+#include "clutter-gst-video-texture.h"
 #include "clutter-gst-private.h"
 #include "clutter-gst-shaders.h"
+
+#ifdef CLUTTER_COGL_HAS_GL
 /* include assembly shaders */
 #include "I420.h"
 #include "YV12.h"
+#endif
 
 #include <gst/gst.h>
 #include <gst/gstvalue.h>
@@ -56,12 +60,22 @@
 #include <glib.h>
 #include <string.h>
 
+/* Flags to give to cogl_texture_new(). Since clutter 1.1.10 put NO_ATLAS to
+ * be sure the frames don't end up in an atlas */
+#if CLUTTER_CHECK_VERSION(1, 1, 10)
+#define CLUTTER_GST_TEXTURE_FLAGS \
+  (COGL_TEXTURE_NO_SLICING | COGL_TEXTURE_NO_ATLAS)
+#else
+#define CLUTTER_GST_TEXTURE_FLAGS  COGL_TEXTURE_NO_SLICING
+#endif
+
 static gchar *ayuv_to_rgba_shader = \
      FRAGMENT_SHADER_VARS
      "uniform sampler2D tex;"
      "void main () {"
      "  vec4 color = texture2D (tex, vec2(" TEX_COORD "));"
      "  float y = 1.1640625 * (color.g - 0.0625);"
+     "  float u = color.b - 0.5;"
      "  float v = color.a - 0.5;"
      "  color.a = color.r;"
      "  color.r = y + 1.59765625 * v;"
@@ -71,11 +85,6 @@ static gchar *ayuv_to_rgba_shader = \
      FRAGMENT_SHADER_END
      "}";
 
-static gchar *dummy_shader = \
-     FRAGMENT_SHADER_VARS
-     "void main () {"
-     "}";
-
 static gchar *yv12_to_rgba_shader = \
      FRAGMENT_SHADER_VARS
      "uniform sampler2D ytex;"
@@ -135,20 +144,6 @@ typedef enum
   CLUTTER_GST_I420,
 } ClutterGstVideoFormat;
 
-typedef void (APIENTRYP GLUNIFORM1IPROC)(GLint location, GLint value);
-/* GL_ARB_fragment_program */
-typedef void (APIENTRYP GLGENPROGRAMSPROC)(GLsizei n, GLuint *programs);
-typedef void (APIENTRYP GLBINDPROGRAMPROC)(GLenum target, GLuint program);
-typedef void (APIENTRYP GLPROGRAMSTRINGPROC)(GLenum target, GLenum format,
-                                             GLsizei len, const void *string);
-typedef struct _ClutterGstSymbols
-{
-  /* GL_ARB_fragment_program */
-  GLGENPROGRAMSPROC   glGenProgramsARB;
-  GLBINDPROGRAMPROC   glBindProgramARB;
-  GLPROGRAMSTRINGPROC glProgramStringARB;
-} ClutterGstSymbols;
-
 /*
  * features: what does the underlaying video card supports ?
  */
@@ -204,11 +199,7 @@ typedef enum _ClutterGstRendererState
 struct _ClutterGstVideoSinkPrivate
 {
   ClutterTexture          *texture;
-  CoglHandle               u_tex;
-  CoglHandle               v_tex;
-  CoglHandle               program;
-  CoglHandle               shader;
-  GLuint                   fp;
+  CoglMaterial            *material_template;
 
   ClutterGstVideoFormat    format;
   gboolean                 bgr;
@@ -217,8 +208,6 @@ struct _ClutterGstVideoSinkPrivate
   int                      fps_n, fps_d;
   int                      par_n, par_d;
   
-  ClutterGstSymbols        syms;          /* extra OpenGL functions */
-
   GMainContext            *clutter_main_context;
   ClutterGstSource        *source;
 
@@ -278,6 +267,7 @@ clutter_gst_source_finalize (GSource *source)
     gst_buffer_unref (gst_source->buffer);
   gst_source->buffer = NULL;
   g_mutex_unlock (gst_source->buffer_lock);
+  g_mutex_free (gst_source->buffer_lock);
 }
 
 static void
@@ -372,6 +362,7 @@ clutter_gst_video_sink_set_priority (ClutterGstVideoSink *sink,
  * Small helpers
  */
 
+#ifdef CLUTTER_COGL_HAS_GL
 static void
 _string_array_to_char_array (char	*dst,
                              const char *src[])
@@ -385,95 +376,108 @@ _string_array_to_char_array (char	*dst,
   }
   *dst = '\0';
 }
-
-static void
-_renderer_connect_signals(ClutterGstVideoSink        *sink,
-                          ClutterGstRendererPaint     paint_func,
-                          ClutterGstRendererPostPaint post_paint_func)
-{
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-  gulong handler_id;
-
-  handler_id =
-    g_signal_connect (priv->texture, "paint", G_CALLBACK (paint_func), sink);
-  g_array_append_val (priv->signal_handler_ids, handler_id);
-
-  handler_id = g_signal_connect_after (priv->texture,
-                                       "paint",
-                                       G_CALLBACK (post_paint_func),
-                                       sink);
-  g_array_append_val (priv->signal_handler_ids, handler_id);
-}
-
-#ifdef CLUTTER_COGL_HAS_GL
-static void
-clutter_gst_video_sink_set_fp_shader (ClutterGstVideoSink *sink,
-                                      const gchar         *shader_src,
-                                      const int            size)
-{
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-
-  /* FIXME: implement freeing the shader */
-  if (!shader_src)
-    return;
-
-  priv->syms.glGenProgramsARB (1, &priv->fp);
-  priv->syms.glBindProgramARB (GL_FRAGMENT_PROGRAM_ARB, priv->fp);
-  priv->syms.glProgramStringARB (GL_FRAGMENT_PROGRAM_ARB,
-                                  GL_PROGRAM_FORMAT_ASCII_ARB,
-                                  size,
-                                  (const GLbyte *)shader_src);
-}
 #endif
 
-static void
-clutter_gst_video_sink_set_glsl_shader (ClutterGstVideoSink *sink,
-                                        const gchar         *shader_src)
+static CoglHandle
+_create_cogl_program (const char *source)
 {
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  CoglHandle shader;
+  CoglHandle program;
   
-  if (priv->texture)
-    clutter_actor_set_shader (CLUTTER_ACTOR (priv->texture), NULL);
-
-  if (priv->program)
-    {
-      cogl_program_unref (priv->program);
-      priv->program = NULL;
-    }
+  /* Create shader through Cogl - necessary as we need to be able to set
+   * integer uniform variables for multi-texturing.
+   */
+  shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
+  cogl_shader_source (shader, source);
+  cogl_shader_compile (shader);
   
-  if (priv->shader)
-    {
-      cogl_shader_unref (priv->shader);
-      priv->shader = NULL;
-    }
-  
-  if (shader_src)
-    {
-      ClutterShader *shader;
+  program = cogl_create_program ();
+  cogl_program_attach_shader (program, shader);
+  cogl_program_link (program);
 
-      /* Set a dummy shader so we don't interfere with the shader stack */
-      shader = clutter_shader_new ();
-      clutter_shader_set_fragment_source (shader, dummy_shader, -1);
-      clutter_actor_set_shader (CLUTTER_ACTOR (priv->texture), shader);
-      g_object_unref (shader);
+  cogl_handle_unref (shader);
 
-      /* Create shader through COGL - necessary as we need to be able to set
-       * integer uniform variables for multi-texturing.
-       */
-      priv->shader = cogl_create_shader (COGL_SHADER_TYPE_FRAGMENT);
-      cogl_shader_source (priv->shader, shader_src);
-      cogl_shader_compile (priv->shader);
-      
-      priv->program = cogl_create_program ();
-      cogl_program_attach_shader (priv->program, priv->shader);
-      cogl_program_link (priv->program);
-    }
+  return program;
 }
 
-/* some renderers don't need all the ClutterGstRenderer vtable */
 static void
-clutter_gst_dummy_init (ClutterGstVideoSink *sink)
+_create_template_material (ClutterGstVideoSink *sink,
+                           const char *source,
+                           gboolean set_uniforms,
+                           int n_layers)
 {
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  CoglMaterial *template;
+  int i;
+  
+  if (priv->material_template)
+    cogl_object_unref (priv->material_template);
+
+  template = cogl_material_new ();
+
+  if (source)
+    {
+      CoglHandle program = _create_cogl_program (source);
+
+      if (set_uniforms)
+        {
+          unsigned int location;
+
+          cogl_program_use (program);
+
+          location = cogl_program_get_uniform_location (program, "ytex");
+          cogl_program_set_uniform_1i (program, location, 0);
+          if (n_layers > 1)
+            {
+              location = cogl_program_get_uniform_location (program, "utex");
+              cogl_program_set_uniform_1i (program, location, 1);
+            }
+          if (n_layers > 2)
+            {
+              location = cogl_program_get_uniform_location (program, "vtex");
+              cogl_program_set_uniform_1i (program, location, 2);
+            }
+
+          cogl_program_use (COGL_INVALID_HANDLE);
+        }
+
+      cogl_material_set_user_program (template, program);
+      cogl_handle_unref (program);
+    }
+
+  for (i = 0; i < n_layers; i++)
+    cogl_material_set_layer (template, i, COGL_INVALID_HANDLE);
+
+  priv->material_template = template;
+}
+
+static void
+_create_paint_material (ClutterGstVideoSink *sink,
+                        CoglHandle tex0,
+                        CoglHandle tex1,
+                        CoglHandle tex2)
+{
+  ClutterGstVideoSinkPrivate *priv= sink->priv;
+  CoglMaterial *material = cogl_material_copy (priv->material_template);
+
+  if (tex0 != COGL_INVALID_HANDLE)
+    {
+      cogl_material_set_layer (material, 0, tex0);
+      cogl_handle_unref (tex0);
+    }
+  if (tex1 != COGL_INVALID_HANDLE)
+    {
+      cogl_material_set_layer (material, 1, tex1);
+      cogl_handle_unref (tex1);
+    }
+  if (tex2 != COGL_INVALID_HANDLE)
+    {
+      cogl_material_set_layer (material, 2, tex2);
+      cogl_handle_unref (tex2);
+    }
+
+  clutter_texture_set_cogl_material (priv->texture, material);
+  cogl_object_unref (material);
 }
 
 static void
@@ -481,6 +485,12 @@ clutter_gst_dummy_deinit (ClutterGstVideoSink *sink)
 {
 }
 
+static void
+clutter_gst_rgb_init (ClutterGstVideoSink *sink)
+{
+  _create_template_material (sink, NULL, FALSE, 1);
+}
+
 /*
  * RGB 24 / BGR 24
  *
@@ -491,18 +501,27 @@ static void
 clutter_gst_rgb24_upload (ClutterGstVideoSink *sink,
                           GstBuffer           *buffer)
 {
-  ClutterGstVideoSinkPrivate *priv= sink->priv;
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  CoglPixelFormat format;
+  CoglHandle tex;
 
-  clutter_texture_set_from_rgb_data (priv->texture,
-                                     GST_BUFFER_DATA (buffer),
-                                     FALSE,
-                                     priv->width,
-                                     priv->height,
-                                     GST_ROUND_UP_4 (3 * priv->width),
-                                     3,
-                                     priv->bgr ?
-                                     CLUTTER_TEXTURE_RGB_FLAG_BGR : 0,
-                                     NULL);
+  if (priv->bgr)
+    format = COGL_PIXEL_FORMAT_BGR_888;
+  else
+    format = COGL_PIXEL_FORMAT_RGB_888;
+
+  tex = cogl_texture_new_from_data (priv->width,
+                                    priv->height,
+                                    CLUTTER_GST_TEXTURE_FLAGS,
+                                    format,
+                                    format,
+                                    GST_ROUND_UP_4 (3 * priv->width),
+                                    GST_BUFFER_DATA (buffer));
+
+  _create_paint_material (sink,
+                          tex,
+                          COGL_INVALID_HANDLE,
+                          COGL_INVALID_HANDLE);
 }
 
 static ClutterGstRenderer rgb24_renderer =
@@ -511,7 +530,7 @@ static ClutterGstRenderer rgb24_renderer =
   CLUTTER_GST_RGB24,
   0,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_RGB ";" GST_VIDEO_CAPS_BGR),
-  clutter_gst_dummy_init,
+  clutter_gst_rgb_init,
   clutter_gst_dummy_deinit,
   clutter_gst_rgb24_upload,
 };
@@ -524,18 +543,27 @@ static void
 clutter_gst_rgb32_upload (ClutterGstVideoSink *sink,
                           GstBuffer           *buffer)
 {
-  ClutterGstVideoSinkPrivate *priv= sink->priv;
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  CoglPixelFormat format;
+  CoglHandle tex;
 
-  clutter_texture_set_from_rgb_data (priv->texture,
-                                     GST_BUFFER_DATA (buffer),
-                                     TRUE,
-                                     priv->width,
-                                     priv->height,
-                                     GST_ROUND_UP_4 (4 * priv->width),
-                                     4,
-                                     priv->bgr ?
-                                     CLUTTER_TEXTURE_RGB_FLAG_BGR : 0,
-                                     NULL);
+  if (priv->bgr)
+    format = COGL_PIXEL_FORMAT_BGRA_8888;
+  else
+    format = COGL_PIXEL_FORMAT_RGBA_8888;
+
+  tex = cogl_texture_new_from_data (priv->width,
+                                    priv->height,
+                                    CLUTTER_GST_TEXTURE_FLAGS,
+                                    format,
+                                    format,
+                                    GST_ROUND_UP_4 (4 * priv->width),
+                                    GST_BUFFER_DATA (buffer));
+
+  _create_paint_material (sink,
+                          tex,
+                          COGL_INVALID_HANDLE,
+                          COGL_INVALID_HANDLE);
 }
 
 static ClutterGstRenderer rgb32_renderer =
@@ -544,7 +572,7 @@ static ClutterGstRenderer rgb32_renderer =
   CLUTTER_GST_RGB32,
   0,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_BGRA),
-  clutter_gst_dummy_init,
+  clutter_gst_rgb_init,
   clutter_gst_dummy_deinit,
   clutter_gst_rgb32_upload,
 };
@@ -552,7 +580,7 @@ static ClutterGstRenderer rgb32_renderer =
 /*
  * YV12
  *
- * 8 bit Y plane followed by 8 bit 2x2 subsampled V and U planes.
+ * 8 bit Y plane followed by 8 bit 2x2 subsampled U and V planes.
  */
 
 static void
@@ -562,106 +590,42 @@ clutter_gst_yv12_upload (ClutterGstVideoSink *sink,
   ClutterGstVideoSinkPrivate *priv = sink->priv;
   gint y_row_stride  = GST_ROUND_UP_4 (priv->width);
   gint uv_row_stride = GST_ROUND_UP_4 (priv->width / 2);
+  CoglHandle y_tex, u_tex, v_tex;
 
-  CoglHandle y_tex = cogl_texture_new_from_data (priv->width,
-                                                 priv->height,
-                                                 COGL_TEXTURE_NO_SLICING,
-                                                 COGL_PIXEL_FORMAT_G_8,
-                                                 COGL_PIXEL_FORMAT_G_8,
-                                                 y_row_stride,
-                                                 GST_BUFFER_DATA (buffer));
+  y_tex = cogl_texture_new_from_data (priv->width,
+                                      priv->height,
+                                      CLUTTER_GST_TEXTURE_FLAGS,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      y_row_stride,
+                                      GST_BUFFER_DATA (buffer));
 
-  clutter_texture_set_cogl_texture (priv->texture, y_tex);
-  cogl_texture_unref (y_tex);
+  u_tex = cogl_texture_new_from_data (priv->width / 2,
+                                      priv->height / 2,
+                                      CLUTTER_GST_TEXTURE_FLAGS,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      uv_row_stride,
+                                      GST_BUFFER_DATA (buffer) +
+                                      (y_row_stride * priv->height));
 
-  if (priv->u_tex)
-    cogl_texture_unref (priv->u_tex);
+  v_tex = cogl_texture_new_from_data (priv->width / 2,
+                                      priv->height / 2,
+                                      CLUTTER_GST_TEXTURE_FLAGS,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      COGL_PIXEL_FORMAT_G_8,
+                                      uv_row_stride,
+                                      GST_BUFFER_DATA (buffer)
+                                      + (y_row_stride * priv->height)
+                                      + (uv_row_stride * priv->height / 2));
 
-  if (priv->v_tex)
-    cogl_texture_unref (priv->v_tex);
-
-  priv->v_tex = cogl_texture_new_from_data (priv->width / 2,
-                                            priv->height / 2,
-                                            COGL_TEXTURE_NO_SLICING,
-                                            COGL_PIXEL_FORMAT_G_8,
-                                            COGL_PIXEL_FORMAT_G_8,
-                                            uv_row_stride,
-                                            GST_BUFFER_DATA (buffer) +
-                                            (y_row_stride * priv->height));
-
-  priv->u_tex =
-    cogl_texture_new_from_data (priv->width / 2,
-                                priv->height / 2,
-                                COGL_TEXTURE_NO_SLICING,
-                                COGL_PIXEL_FORMAT_G_8,
-                                COGL_PIXEL_FORMAT_G_8,
-                                uv_row_stride,
-                                GST_BUFFER_DATA (buffer)
-                                + (y_row_stride * priv->height)
-                                + (uv_row_stride * priv->height / 2));
-}
-
-static void
-clutter_gst_yv12_glsl_paint (ClutterActor        *actor,
-                             ClutterGstVideoSink *sink)
-{
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-  CoglHandle material;
-
-  material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (actor));
-
-  /* bind the shader */
-  cogl_program_use (priv->program);
-
-  /* Bind the U and V textures in layers 1 and 2 */
-  if (priv->u_tex)
-    cogl_material_set_layer (material, 1, priv->u_tex);
-  if (priv->v_tex)
-    cogl_material_set_layer (material, 2, priv->v_tex);
-}
-
-static void
-clutter_gst_yv12_glsl_post_paint (ClutterActor        *actor,
-                                  ClutterGstVideoSink *sink)
-{
-  CoglHandle material;
-
-  /* Remove the extra layers */
-  material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (actor));
-  cogl_material_remove_layer (material, 1);
-  cogl_material_remove_layer (material, 2);
-
-  /* disable the shader */
-  cogl_program_use (COGL_INVALID_HANDLE);
+  _create_paint_material (sink, y_tex, u_tex, v_tex);
 }
 
 static void
 clutter_gst_yv12_glsl_init (ClutterGstVideoSink *sink)
 {
-  ClutterGstVideoSinkPrivate *priv= sink->priv;
-  GLint location;
-
-  clutter_gst_video_sink_set_glsl_shader (sink, yv12_to_rgba_shader);
-
-  cogl_program_use (priv->program);
-  location = cogl_program_get_uniform_location (priv->program, "ytex");
-  cogl_program_uniform_1i (location, 0);
-  location = cogl_program_get_uniform_location (priv->program, "utex");
-  cogl_program_uniform_1i (location, 1);
-  location = cogl_program_get_uniform_location (priv->program, "vtex");
-  cogl_program_uniform_1i (location, 2);
-
-  cogl_program_use (COGL_INVALID_HANDLE);
-
-  _renderer_connect_signals (sink,
-                             clutter_gst_yv12_glsl_paint,
-                             clutter_gst_yv12_glsl_post_paint);
-}
-
-static void
-clutter_gst_yv12_glsl_deinit (ClutterGstVideoSink *sink)
-{
-  clutter_gst_video_sink_set_glsl_shader (sink, NULL);
+  _create_template_material (sink, yv12_to_rgba_shader, TRUE, 3);
 }
 
 
@@ -672,7 +636,7 @@ static ClutterGstRenderer yv12_glsl_renderer =
   CLUTTER_GST_GLSL | CLUTTER_GST_MULTI_TEXTURE,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("YV12")),
   clutter_gst_yv12_glsl_init,
-  clutter_gst_yv12_glsl_deinit,
+  clutter_gst_dummy_deinit,
   clutter_gst_yv12_upload,
 };
 
@@ -683,78 +647,15 @@ static ClutterGstRenderer yv12_glsl_renderer =
  */
 
 #ifdef CLUTTER_COGL_HAS_GL
-static void
-clutter_gst_yv12_fp_paint (ClutterActor        *actor,
-                             ClutterGstVideoSink *sink)
-{
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-  CoglHandle material;
-
-  material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (actor));
-
-  /* Bind the U and V textures in layers 1 and 2 */
-  if (priv->u_tex)
-    cogl_material_set_layer (material, 1, priv->u_tex);
-  if (priv->v_tex)
-    cogl_material_set_layer (material, 2, priv->v_tex);
-
-  /* Cogl doesn't support changing OpenGL state to modify how Cogl primitives
-   * work, but it also doesn't support ARBfp which we currently depend on. For
-   * now we at least ask Cogl to flush any batched primitives so we avoid
-   * binding our shader across the wrong geometry, but there is a risk that
-   * Cogl may start to use ARBfp internally which will conflict with us. */
-  cogl_flush ();
-
-  /* bind the shader */
-  glEnable (GL_FRAGMENT_PROGRAM_ARB);
-  priv->syms.glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, priv->fp);
-
-}
-
-static void
-clutter_gst_yv12_fp_post_paint (ClutterActor        *actor,
-                                ClutterGstVideoSink *sink)
-{
-  CoglHandle material;
-
-  /* Cogl doesn't support changing OpenGL state to modify how Cogl primitives
-   * work, but it also doesn't support ARBfp which we currently depend on. For
-   * now we at least ask Cogl to flush any batched primitives so we avoid
-   * binding our shader across the wrong geometry, but there is a risk that
-   * Cogl may start to use ARBfp internally which will conflict with us. */
-  cogl_flush ();
-
-  /* Remove the extra layers */
-  material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE (actor));
-  cogl_material_remove_layer (material, 1);
-  cogl_material_remove_layer (material, 2);
-
-  /* Disable the shader */
-  glDisable (GL_FRAGMENT_PROGRAM_ARB);
-}
-
 static void
 clutter_gst_yv12_fp_init (ClutterGstVideoSink *sink)
 {
-  gchar *shader;
-
-  shader = g_malloc(YV12_FP_SZ + 1);
+  char *shader = g_malloc (YV12_FP_SZ + 1);
   _string_array_to_char_array (shader, YV12_fp);
 
-  /* the size given to glProgramStringARB is without the trailing '\0', which
-   * is precisely YV12_FP_SZ */
-  clutter_gst_video_sink_set_fp_shader (sink, shader, YV12_FP_SZ);
-  g_free(shader);
+  _create_template_material (sink, shader, FALSE, 3);
 
-  _renderer_connect_signals (sink,
-                             clutter_gst_yv12_fp_paint,
-                             clutter_gst_yv12_fp_post_paint);
-}
-
-static void
-clutter_gst_yv12_fp_deinit (ClutterGstVideoSink *sink)
-{
-  clutter_gst_video_sink_set_fp_shader (sink, NULL, 0);
+  g_free (shader);
 }
 
 static ClutterGstRenderer yv12_fp_renderer =
@@ -764,7 +665,7 @@ static ClutterGstRenderer yv12_fp_renderer =
   CLUTTER_GST_FP | CLUTTER_GST_MULTI_TEXTURE,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("YV12")),
   clutter_gst_yv12_fp_init,
-  clutter_gst_yv12_fp_deinit,
+  clutter_gst_dummy_deinit,
   clutter_gst_yv12_upload,
 };
 #endif
@@ -779,23 +680,7 @@ static ClutterGstRenderer yv12_fp_renderer =
 static void
 clutter_gst_i420_glsl_init (ClutterGstVideoSink *sink)
 {
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
-  GLint location;
-
-  clutter_gst_video_sink_set_glsl_shader (sink, yv12_to_rgba_shader);
-
-  cogl_program_use (priv->program);
-  location = cogl_program_get_uniform_location (priv->program, "ytex");
-  cogl_program_uniform_1i (location, 0);
-  location = cogl_program_get_uniform_location (priv->program, "vtex");
-  cogl_program_uniform_1i (location, 1);
-  location = cogl_program_get_uniform_location (priv->program, "utex");
-  cogl_program_uniform_1i (location, 2);
-  cogl_program_use (COGL_INVALID_HANDLE);
-
-  _renderer_connect_signals (sink,
-                             clutter_gst_yv12_glsl_paint,
-                             clutter_gst_yv12_glsl_post_paint);
+  _create_template_material (sink, yv12_to_rgba_shader, TRUE, 3);
 }
 
 static ClutterGstRenderer i420_glsl_renderer =
@@ -805,7 +690,7 @@ static ClutterGstRenderer i420_glsl_renderer =
   CLUTTER_GST_GLSL | CLUTTER_GST_MULTI_TEXTURE,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420")),
   clutter_gst_i420_glsl_init,
-  clutter_gst_yv12_glsl_deinit,
+  clutter_gst_dummy_deinit,
   clutter_gst_yv12_upload,
 };
 
@@ -820,19 +705,12 @@ static ClutterGstRenderer i420_glsl_renderer =
 static void
 clutter_gst_i420_fp_init (ClutterGstVideoSink *sink)
 {
-  gchar *shader;
-
-  shader = g_malloc(I420_FP_SZ + 1);
+  char *shader = g_malloc(I420_FP_SZ + 1);
   _string_array_to_char_array (shader, I420_fp);
 
-  /* the size given to glProgramStringARB is without the trailing '\0', which
-   * is precisely I420_FP_SZ */
-  clutter_gst_video_sink_set_fp_shader (sink, shader, I420_FP_SZ);
-  g_free(shader);
+  _create_template_material (sink, shader, FALSE, 3);
 
-  _renderer_connect_signals (sink,
-                             clutter_gst_yv12_fp_paint,
-                             clutter_gst_yv12_fp_post_paint);
+  g_free (shader);
 }
 
 static ClutterGstRenderer i420_fp_renderer =
@@ -842,7 +720,7 @@ static ClutterGstRenderer i420_fp_renderer =
   CLUTTER_GST_FP | CLUTTER_GST_MULTI_TEXTURE,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420")),
   clutter_gst_i420_fp_init,
-  clutter_gst_yv12_fp_deinit,
+  clutter_gst_dummy_deinit,
   clutter_gst_yv12_upload,
 };
 #endif
@@ -858,30 +736,27 @@ static ClutterGstRenderer i420_fp_renderer =
 static void
 clutter_gst_ayuv_glsl_init(ClutterGstVideoSink *sink)
 {
-  clutter_gst_video_sink_set_glsl_shader (sink, ayuv_to_rgba_shader);
-}
-
-static void
-clutter_gst_ayuv_glsl_deinit(ClutterGstVideoSink *sink)
-{
-  clutter_gst_video_sink_set_glsl_shader (sink, NULL);
+  _create_template_material (sink, ayuv_to_rgba_shader, TRUE, 1);
 }
 
 static void
 clutter_gst_ayuv_upload (ClutterGstVideoSink *sink,
                          GstBuffer           *buffer)
 {
-  ClutterGstVideoSinkPrivate *priv= sink->priv;
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  CoglHandle tex =
+    cogl_texture_new_from_data (priv->width,
+                                priv->height,
+                                CLUTTER_GST_TEXTURE_FLAGS,
+                                COGL_PIXEL_FORMAT_RGBA_8888,
+                                COGL_PIXEL_FORMAT_RGBA_8888,
+                                GST_ROUND_UP_4 (4 * priv->width),
+                                GST_BUFFER_DATA (buffer));
 
-  clutter_texture_set_from_rgb_data (priv->texture,
-                                     GST_BUFFER_DATA (buffer),
-                                     TRUE,
-                                     priv->width,
-                                     priv->height,
-                                     GST_ROUND_UP_4 (4 * priv->width),
-                                     4,
-                                     0,
-                                     NULL);
+  _create_paint_material (sink,
+                          tex,
+                          COGL_INVALID_HANDLE,
+                          COGL_INVALID_HANDLE);
 }
 
 static ClutterGstRenderer ayuv_glsl_renderer =
@@ -891,15 +766,14 @@ static ClutterGstRenderer ayuv_glsl_renderer =
   CLUTTER_GST_GLSL,
   GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV")),
   clutter_gst_ayuv_glsl_init,
-  clutter_gst_ayuv_glsl_deinit,
+  clutter_gst_dummy_deinit,
   clutter_gst_ayuv_upload,
 };
 
 static GSList *
-clutter_gst_build_renderers_list (ClutterGstSymbols *syms)
+clutter_gst_build_renderers_list (void)
 {
   GSList             *list = NULL;
-  const gchar        *gl_extensions;
   GLint               nb_texture_units = 0;
   gint                features = 0, i;
   /* The order of the list of renderers is important. They will be prepended
@@ -920,34 +794,14 @@ clutter_gst_build_renderers_list (ClutterGstSymbols *syms)
       NULL
     };
 
-  /* get the features */
-  gl_extensions = (const gchar*) glGetString (GL_EXTENSIONS);
-
   glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &nb_texture_units);
 
   if (nb_texture_units >= 3)
     features |= CLUTTER_GST_MULTI_TEXTURE;
 
 #ifdef CLUTTER_COGL_HAS_GL
-  if (cogl_check_extension ("GL_ARB_fragment_program", gl_extensions))
-    {
-      /* the shaders we'll feed to the GPU are simple enough, we don't need
-       * to check GL limits for GL_FRAGMENT_PROGRAM_ARB */
-
-      syms->glGenProgramsARB = (GLGENPROGRAMSPROC)
-        cogl_get_proc_address ("glGenProgramsARB");
-      syms->glBindProgramARB = (GLBINDPROGRAMPROC)
-        cogl_get_proc_address ("glBindProgramARB");
-      syms->glProgramStringARB = (GLPROGRAMSTRINGPROC)
-        cogl_get_proc_address ("glProgramStringARB");
-
-      if (syms->glGenProgramsARB &&
-          syms->glBindProgramARB &&
-          syms->glProgramStringARB)
-        {
-          features |= CLUTTER_GST_FP;
-        }
-    }
+  if (cogl_features_available (COGL_FEATURE_SHADERS_ARBFP))
+    features |= CLUTTER_GST_FP;
 #endif
 
   if (cogl_features_available (COGL_FEATURE_SHADERS_GLSL))
@@ -1039,8 +893,7 @@ clutter_gst_video_sink_init (ClutterGstVideoSink      *sink,
    * the clutter thread)  */
   priv->clutter_main_context = g_main_context_default ();
 
-
-  priv->renderers = clutter_gst_build_renderers_list (&priv->syms);
+  priv->renderers = clutter_gst_build_renderers_list ();
   priv->caps = clutter_gst_build_caps (priv->renderers);
   priv->renderer_state = CLUTTER_GST_RENDERER_STOPPED;
 
@@ -1064,7 +917,7 @@ clutter_gst_video_sink_get_caps (GstBaseSink *bsink)
   ClutterGstVideoSink *sink;
 
   sink = CLUTTER_GST_VIDEO_SINK (bsink);
-  return gst_caps_copy (sink->priv->caps);
+  return gst_caps_ref (sink->priv->caps);
 }
 
 static gboolean
@@ -1118,6 +971,17 @@ clutter_gst_video_sink_set_caps (GstBaseSink *bsink,
   else 
     priv->par_n = priv->par_d = 1;
 
+  /* If we happen to use a ClutterGstVideoTexture, now is to good time to
+   * instruct it about the pixel aspect ratio so we can have a correct
+   * natural width/height */
+  if (CLUTTER_GST_IS_VIDEO_TEXTURE (priv->texture))
+    {
+      ClutterGstVideoTexture *texture =
+        (ClutterGstVideoTexture *) priv->texture;
+
+      _clutter_gst_video_texture_set_par (texture, priv->par_n, priv->par_d);
+    }
+
   ret = gst_structure_get_fourcc (structure, "format", &fourcc);
   if (ret && (fourcc == GST_MAKE_FOURCC ('Y', 'V', '1', '2')))
     {
